function H = shermfuji(x,y)

%SHERMFUJI      Hermite-Fujiwara matrix with symbolic or numerical coefficients 
% of a polynomial
%
% The instruction
%
%   H = SHERMFUJI(P)
%
% computes the Hermite-Fujiwara matrix of continuous-time scalar 
% polynomial P with numerical coefficients. The polynomial P is asymtotically 
% stable if and only if the Hermite-Fujiwara matrix H is positive definite.
%
% The instruction
%
%   H = SHERMFUJI(P,Q)
%
% computes the Hermite-Fujiwara matrix of continuous-time scalar 
% polynomial P with symbolic coefficients.
% A second argument Q may be specified in which variable is polynomial P.
%
% See also: HERMFUJI, SCHURCOHN, SSCHURCOHN.
%           MATDEG, MATCOEF

%    Author: Petr Urban May, 24, 2002.


%-----<<< Parse input arguments >>>-----
if nargin<1
    error('Not enough input arguments.');
end

if isa(class(x),'sym') | isa(class(x),'double')
    error('Input must be of class sym');
end

if nargin == 1
    sp = symbol(x);
    if ~(isempty(sp) | strcmp(sp,'s') | strcmp(sp,'p'))
       error('Continuous-time polynomials only.');
    end
    p = deg(x);
    q = pol2mat(x);
else
    if ~((char(y) == 's') | (char(y) == 'p'))
       error('Continuous-time polynomials only.');
    end
    p = matdeg(x,y);
    q = matcoef(x,y);
    q = fliplr(q);
end
r = zeros(size(q));
q = [q r];

%-----<<< Compute the Hermite-Fujiwara matrix >>>-----
for j = 1:p
    for i = 1:j
        hij = 0;
        if (mod(i+j,2) == 0)
            for k = 1:j
                hij = hij + ((-1)^(k+i-2))*q(k)*q(i+j-k+1);
            end
        end
        H(i,j) = hij;
        H(j,i) = H(i,j);
    end
end

%-----<<< END >>>-----