function S = sschurcohn(x,y)
%SSCHURCOHN     Schur-Cohn matrix with symbolic or numerical coefficients 
% of a polynomial
%
% The instruction
%
%   S = SSCHURCOHN(P)
%
% computes the symmetric Schur-Cohn matrix of discrete-time scalar
% polynomial P with numerical coefficients. Matrix S is positive definite 
% if and only if P has all its zeros within the unit disk.
% 
% The instruction
%
%   S = SSCHURCOHN(P,Q)
%
% computes the symmetric Schur-Cohn matrix of discrete-time scalar
% polynomial P with symbolic coefficients.
% A second argument Q may be specified in which variable is polynomial P.
%
% See also: HERMFUJI, SHERMFUJI, SCHURCOHN
%           MATDEG, MATCOEF

%    Author: P. Urban May 25, 2002.


%-----<<< Parse input arguments >>>-----
if nargin<1
    error('Not enough input arguments.');
end

if isa(class(x),'sym') | isa(class(x),'double')
    error('Input must be of class sym or double');
end

if nargin == 1
    sp = symbol(x);
    if ~(isempty(sp) | strcmp(sp,'z') | strcmp(sp,'z^-1') | ...
          strcmp(sp,'d') | strcmp(sp,'q'))
       error('Discrete-time polynomial only.');
    end
    p = deg(x);
    q = pol2mat(x);
else
    if ~((char(y) == 'z') | (char(y) == 'z^-1') | ...
          (char(y) == 'd') | (char(y) == 'q'))
       error('Discrete-time polynomial only.');
    end
    p = matdeg(x,y);
    q = matcoef(x,y);
    q = fliplr(q);
end
r = zeros(size(q));
q = [q r];

%-----<<< Compute the Schur-Cohn matrix >>>-----
for j = 1:p
    for i = 1:j
        sij = 0;
        for r = 1:i
            sij = sij + (q(i+1-r)*q(j+1-r) - q(p+r-i+1)*q(p+r-j+1));
        end
        S(i,j) = sij;
        S(j,i) = S(i,j);
    end
end

%-----<<< END >>>-----